@@ -136,7 +136,7 @@ class Colormap(str, Enum):
 
 
 class ProcessingStepAttribute(MappingAttribute):
-    """File definition for an Image.
+    """Processing Levels Step.
 
     Attributes
     ----------
@@ -159,8 +159,8 @@ class ProcessingLevelsAttribute(ModelAttribute, MutableMapping):
 
     Can be set using a dictionary of items or any `Mapping`, or an instance of this
     attribute.  All keys must be string and values can be string or an iterable
-    of ProcessingStepAttributes (or compatible mapping).
-    ProcessingLevelsAttribute behaves similar to dictionaries.
+    of `ProcessingStepAttribute` items (or compatible mapping).
+    `ProcessingLevelsAttribute` behaves similar to dictionaries.
     """
 
     # this value is ONLY used for for instances of the attribute that
@@ -197,10 +197,11 @@ class ProcessingLevelsAttribute(ModelAttribute, MutableMapping):
     def validate_key_and_value(self, key, value):
         """Validate the key and value.
 
-        The key must be a string, and the value either a string or an iterable of ProcessingStepAttributes
-        or a compatible mapping.
+        The key must be a string, and the value either a string or an iterable of
+        `ProcessingStepAttribute` items or a compatible mapping.
 
-        Returns a fully formed value (a string or a ListAttribute of ProcessingStepAttributes)
+        Returns a fully formed value (a string or a ListAttribute of
+        `ProcessingStepAttribute` items)
         """
         if not isinstance(key, str):
             raise AttributeValidationError(
@@ -266,8 +267,8 @@ class ProcessingLevelsAttribute(ModelAttribute, MutableMapping):
 
         Parameters
         ----------
-        value : dict or ProcessingLevelsAttribute
-            A set of values to use to initialize a new ProcessingLevelsAttribute
+        value : dict or `ProcessingLevelsAttribute`
+            A set of values to use to initialize a new `ProcessingLevelsAttribute`
             instance.  All keys must be strings, and values can be strings or numbers.
 
         Returns
@@ -334,6 +335,58 @@ class ProcessingLevelsAttribute(ModelAttribute, MutableMapping):
         return len(self._items)
 
 
+class DerivedParamsAttribute(MappingAttribute):
+    """Derived Band Parameters Attribute.
+
+    Attributes
+    ----------
+    function : str
+        Name of the function to apply. Required.
+    bands : list(str)
+        Names of the bands used as input to the function. Required.
+    source_type : int
+        Optional index into the named parameter (an array) for the band.
+    """
+
+    function = TypedAttribute(str)
+    bands = ListAttribute(TypedAttribute(str), validate=True)
+    source_type = EnumAttribute(
+        DataType,
+        doc="str or DataType: The datatype for extracting pixels from the source bands.",
+    )
+
+    def deserialize(self, value, validate=True):
+        """Deserialize a value to a native type.
+
+        See :meth:`Attribute.deserialize`.
+
+        Parameters
+        ----------
+        values : dict or MappingAttribute
+            The values to use to initialize a new MappingAttribute.
+
+        Returns
+        -------
+        DerivedParamsAttribute
+            A `DerivedParamsAttribute` instance with the given values.
+
+        Raises
+        ------
+        AttributeValidationError
+            If the value is not a `DerivedParamsAttribute` or a mapping with
+            a `function`, `bands`, and (optionally) `source_type key.
+        """
+        result = super(DerivedParamsAttribute, self).deserialize(value, validate)
+
+        if result:
+            if not result.function:
+                raise AttributeValidationError("'function' field required.")
+            if not result.bands:
+                raise AttributeValidationError("'bands' field required.")
+
+        return result
+
+
 class Band(NamedCatalogObject):
     """A data band in images of a specific product.
 
@@ -357,6 +410,7 @@ class Band(NamedCatalogObject):
     :attr:`~descarteslabs.catalog.GenericBand.band_index`,
     :attr:`~descarteslabs.catalog.GenericBand.file_index`,
     :attr:`~descarteslabs.catalog.GenericBand.jpx_layer_index`.
+    :attr:`~descarteslabs.catalog.GenericBand.vendor_band_name`.
 
     To create a new band instantiate one of those specialized classes:
 
@@ -367,7 +421,9 @@ class Band(NamedCatalogObject):
       :attr:`~SpectralBand.wavelength_nm_center`,
       :attr:`~SpectralBand.wavelength_nm_min`,
       :attr:`~SpectralBand.wavelength_nm_max`,
-      :attr:`~SpectralBand.wavelength_nm_fwhm`.
+      :attr:`~SpectralBand.wavelength_nm_fwhm`,
+      :attr:`~descarteslabs.catalog.GenericBand.processing_levels`,
+      :attr:`~descarteslabs.catalog.GenericBand.derived_params`.
     * `MicrowaveBand`: A band that lies in the microwave spectrum, often from SAR or
       passive radar sensors. Specific attributes:
       :attr:`~MicrowaveBand.frequency`,
@@ -376,7 +432,8 @@ class Band(NamedCatalogObject):
       :attr:`~MicrowaveBand.physical_range_unit`.
     * `MaskBand`: A binary band where by convention a 0 means masked and 1 means
       non-masked. The :attr:`~Band.data_range` and :attr:`~Band.display_range` for
-      masks is implicitly ``[0, 1]``. Specific attributes::attr:`~MaskBand.is_alpha`.
+      masks is implicitly ``[0, 1]``. Specific attributes:
+      :attr:`~MaskBand.is_alpha`.
     * `ClassBand`: A band that maps a finite set of values that may not be continuous to
       classification categories (e.g. a land use classification). A visualization with
       straight pixel values is typically not useful, so commonly a
@@ -385,8 +442,13 @@ class Band(NamedCatalogObject):
       :attr:`~ClassBand.class_labels`.
     * `GenericBand`: A generic type for bands that are not represented by the other band
       types, e.g., mapping physical values like temperature or angles. Specific
-      attributes: :attr:`~GenericBand.colormap`, :attr:`~GenericBand.colormap_name`,
-      :attr:`~GenericBand.physical_range`, :attr:`~GenericBand.physical_range_unit`.
+      attributes:
+      :attr:`~GenericBand.colormap`,
+      :attr:`~GenericBand.colormap_name`,
+      :attr:`~GenericBand.physical_range`,
+      :attr:`~GenericBand.physical_range_unit`,
+      :attr:`~descarteslabs.catalog.GenericBand.processing_levels`,
+      :attr:`~descarteslabs.catalog.GenericBand.derived_params`.
     """
 
     _DOC_DESCRIPTION = """A description with further details on the band.
@@ -451,9 +513,8 @@ class Band(NamedCatalogObject):
     vendor_order = TypedAttribute(
         int,
         doc="""int, optional: A number defining the ordering of bands within a product
-        as defined by the data vendor. 1-based. Used for indexing ``c6s_dlsr``,
-        ``brightness_temperature_k1_k2``, ``radiance_gain_bias``, ``reflectance_gain_bias``,
-        and ``temperature_gain_bias``. Generally only used internally by certain core products.
+        as defined by the data vendor. 1-based. Used for indexing ``c6s_dlsr``.
+        Generally only used internally by certain core products.
 
         *Sortable*.
         """,
@@ -503,7 +564,15 @@ class Band(NamedCatalogObject):
         Defaults to 0.
         """,
     )
+    vendor_band_name = TypedAttribute(
+        str,
+        doc="""str, optional: The name of the band in the source file.
+
+        Some source file types require that the band be indexed by name rather than by the ``band_index``.
+        """,
+    )
     processing_levels = ProcessingLevelsAttribute()
+    derived_params = DerivedParamsAttribute()
 
     def __new__(cls, *args, **kwargs):
         return _new_abstract_class(cls, Band)
