@@ -159,6 +159,7 @@ class CatalogObjectBase(AttributeEqualityMixin):
 
         self._attributes = {}
         self._modified = set()
+        self._deleted = False
 
         self._initialize(
             id=kwargs.pop("id", None),
@@ -474,7 +475,7 @@ class CatalogObjectBase(AttributeEqualityMixin):
 
         Raises
         ------
-        ClientError or ServerError
+        ~descarteslabs.client.exceptions.ClientError or ~descarteslabs.client.exceptions.ServerError
             :ref:`Spurious exception <network_exceptions>` that can occur during a
             network request.
         """
@@ -578,7 +579,7 @@ class CatalogObjectBase(AttributeEqualityMixin):
         NotFoundError
             If any of the requested objects do not exist in the Descartes Labs catalog
             and `ignore_missing` is ``False``.
-        ClientError or ServerError
+        ~descarteslabs.client.exceptions.ClientError or ~descarteslabs.client.exceptions.ServerError
             :ref:`Spurious exception <network_exceptions>` that can occur during a
             network request.
         """
@@ -642,7 +643,7 @@ class CatalogObjectBase(AttributeEqualityMixin):
 
         Raises
         ------
-        ClientError or ServerError
+        ~descarteslabs.client.exceptions.ClientError or ~descarteslabs.client.exceptions.ServerError
             :ref:`Spurious exception <network_exceptions>` that can occur during a
             network request.
         """
@@ -725,7 +726,7 @@ class CatalogObjectBase(AttributeEqualityMixin):
             If any of the attribute values are invalid.
         DeletedObjectError
             If this catalog object was deleted.
-        ClientError or ServerError
+        ~descarteslabs.client.exceptions.ClientError or ~descarteslabs.client.exceptions.ServerError
             :ref:`Spurious exception <network_exceptions>` that can occur during a
             network request.
 
@@ -805,7 +806,7 @@ class CatalogObjectBase(AttributeEqualityMixin):
             If the catalog object is not in the ``SAVED`` state.
         DeletedObjectError
             If this catalog object was deleted.
-        ClientError or ServerError
+        ~descarteslabs.client.exceptions.ClientError or ~descarteslabs.client.exceptions.ServerError
             :ref:`Spurious exception <network_exceptions>` that can occur during a
             network request.
 
@@ -875,7 +876,7 @@ class CatalogObjectBase(AttributeEqualityMixin):
         ------
         ConflictError
             If the object has related objects (bands, images) that exist.
-        ClientError or ServerError
+        ~descarteslabs.client.exceptions.ClientError or ~descarteslabs.client.exceptions.ServerError
             :ref:`Spurious exception <network_exceptions>` that can occur during a
             network request.
 
@@ -905,7 +906,7 @@ class CatalogObjectBase(AttributeEqualityMixin):
             If this catalog object was already deleted.
         UnsavedObjectError
             If this catalog object is being deleted without having been saved.
-        ClientError or ServerError
+        ~descarteslabs.client.exceptions.ClientError or ~descarteslabs.client.exceptions.ServerError
             :ref:`Spurious exception <network_exceptions>` that can occur during a
             network request.
         """
