@@ -20,9 +20,7 @@ import struct
 import time
 
 from tqdm import tqdm
-from urllib3.exceptions import ProtocolError
-from urllib3.util.retry import Retry
-from requests.adapters import HTTPAdapter
+from urllib3.exceptions import ProtocolError, IncompleteRead
 from PIL import Image
 
 from descarteslabs.client.addons import blosc, concurrent, numpy as np
@@ -30,7 +28,6 @@ from descarteslabs.client.auth import Auth
 from descarteslabs.client.services.places import Places
 from descarteslabs.client.services.service.service import Service
 from descarteslabs.client.exceptions import ServerError
-from descarteslabs.common.threading.local import ThreadLocalWrapper
 from descarteslabs.common.dltile import Tile
 
 from .geotiff_utils import make_geotiff
@@ -129,7 +126,7 @@ def read_tiled_blosc_array(metadata, data, progress=None):
     return output
 
 
-def yield_chunks(metadata, data, progress):
+def yield_chunks(metadata, data, progress, nodata):
     dtype = np.dtype(metadata["dtype"])
     chunk_iter = range(metadata["chunks"])
     if progress:
@@ -154,8 +151,17 @@ def yield_chunks(metadata, data, progress):
             )
         blosc.decompress_ptr(buffer, chunk.__array_interface__["data"][0])
 
-        # read mask chunk off response but don't do anything
-        _ = read_blosc_buffer(data)
+        mask_raw_size, mask_buffer = read_blosc_buffer(data)
+        if nodata is not None:
+            mask_chunk = np.ma.empty(chunk_metadata["shape"], dtype=np.bool)
+            if mask_raw_size != mask_chunk.nbytes:
+                raise ServerError(
+                    "Did not receive complete mask chunk (got {}, expected {})".format(
+                        raw_size, chunk.nbytes
+                    )
+                )
+            blosc.decompress_ptr(mask_buffer, mask_chunk.__array_interface__["data"][0])
+            chunk[mask_chunk] = nodata
 
         if chunk.shape[0] == 1:
             yield np.squeeze(chunk, axis=0)
@@ -164,6 +170,7 @@ def yield_chunks(metadata, data, progress):
 
 
 def _retry(req, headers=None):
+    # this provides a nominal 60 seconds of retry
     DELAY = 0.5
     MULTIPLIER = 2
     JITTER = 1.0
@@ -181,15 +188,14 @@ def _retry(req, headers=None):
 
         try:
             return req(headers=headers)
-        except ProtocolError:
-            # generally an incomplete read, perhaps a timeout
-            if retry_count == MAX_RETRIES:
-                raise
-        except ServerError:
-            # can also arise from incomplete reads, or from server-side
-            # error while streaming, or the usual bad status
+        except (IncompleteRead, ProtocolError, ServerError):
+            # IncompleteRead: Response length doesn’t match expected Content-Length
+            # ProtocolError: Something unexpected happened mid-request/response
+            # ServerError: the usual retryable bad status >= 500. Normally won't
+            # occur thanks to the client Retry configuration.
             if retry_count == MAX_RETRIES:
                 raise
+        # MaxRetry and all other ClientError types will be raised to our caller
 
         if retry_count:
             # see https://aws.amazon.com/blogs/architecture/exponential-backoff-and-jitter/
@@ -212,23 +218,6 @@ class Raster(Service):
 
     TIMEOUT = (CONNECT_TIMEOUT, READ_TIMEOUT)
 
-    # override the usual Service retry behavior, as calls will be wrapped by the _retry method.
-    RETRY_CONFIG = Retry(
-        total=10,
-        # NOTE: multiplied by 2 ** num_retries, only used for >=2nd retry (first has no delay)
-        # https://urllib3.readthedocs.io/en/latest/reference/urllib3.util.html#urllib3.util.retry.Retry
-        backoff_factor=random.uniform(1, 3),
-        # By default, POST requests are not retried
-        method_whitelist=frozenset(
-            ["HEAD", "TRACE", "GET", "POST", "PUT", "OPTIONS", "DELETE"]
-        ),
-        # Don't retry here on bad status codes
-        status=0,
-    )
-
-    # without this, the service would keep using the base Service.RETRY_CONFIG
-    ADAPTER = ThreadLocalWrapper(lambda: HTTPAdapter(max_retries=Raster.RETRY_CONFIG))
-
     def __init__(self, url=None, auth=None):
         """The parent Service class implements authentication and exponential
         backoff/retry. Override the url parameter to use a different instance
@@ -266,6 +255,7 @@ class Raster(Service):
         outfile_basename=None,
         headers=None,
         progress=None,
+        nodata=None,
         _retry=_retry,
         **pass_through_params,
     ):
@@ -318,6 +308,8 @@ class Raster(Service):
             reflectance algorithm to the output.
         :param str outfile_basename: Overrides default filename using this string as a base.
         :param bool progress: Display a progress bar.
+        :param None or number: A nodata value to use in the file where pixels are masked.
+            Only used for non-JPEG geotiff files.
 
         :return: A tuple of (`filename`, ``metadata`` dictionary).
             The dictionary contains details about the raster operation that happened.
@@ -367,20 +359,26 @@ class Raster(Service):
             metadata = json.loads(r.raw.readline().decode("utf-8").strip())
             blosc_meta = json.loads(r.raw.readline().decode("utf-8").strip())
 
-            chunk_iter = yield_chunks(blosc_meta, r.raw, progress)
+            chunk_iter = yield_chunks(blosc_meta, r.raw, progress, nodata)
 
             if "id" not in metadata:
                 metadata["id"] = inputs[0]
 
             try:
                 if output_format == "GTiff":
-                    make_geotiff(outfile, chunk_iter, metadata, blosc_meta, None)
+                    make_geotiff(
+                        outfile, chunk_iter, metadata, blosc_meta, None, nodata
+                    )
                 elif output_format == "JPEG":
-                    make_geotiff(outfile, chunk_iter, metadata, blosc_meta, "JPEG")
+                    make_geotiff(
+                        outfile, chunk_iter, metadata, blosc_meta, "JPEG", None
+                    )
                 elif output_format == "PNG":
                     tif_out = outfile_basename + ".tif"
                     try:
-                        make_geotiff(tif_out, chunk_iter, metadata, blosc_meta, "PNG")
+                        make_geotiff(
+                            tif_out, chunk_iter, metadata, blosc_meta, "PNG", None
+                        )
                         try:
                             im = Image.open(tif_out)
                             im.save(outfile)
@@ -783,17 +781,7 @@ class Raster(Service):
 
     def dltile(self, key):
         """
-        Given a DLTile key, return a DLTile GeoJSON Feature.
-        Deprecated for descarteslabs.scenes.DLTile.from_key.
-
-        :param str key: A DLTile key that identifies a DLTile
-
-        :return: A DLTile GeoJSON Feature
-        :rtype: DotDict
-
-        :raises descarteslabs.client.exceptions.BadRequestError: if the given key
-            is not a valid DLTile key
-
+        This method has been removed and is no longer included in the documentation.
         """
         raise NotImplementedError(
             "dltile is removed, use descarteslabs.scenes.DLTile.from_key instead"
@@ -801,17 +789,7 @@ class Raster(Service):
 
     def dltile_from_latlon(self, lat, lon, resolution, tilesize, pad):
         """
-        Return a DLTile GeoJSON Feature that covers a latitude/longitude.
-        Deprecated for descarteslabs.scenes.DLTile.from_latlon.
-
-        :param float lat: Requested latitude
-        :param float lon: Requested longitude
-        :param float resolution: Resolution of DLTile
-        :param int tilesize: Number of valid pixels per DLTile
-        :param int pad: Number of ghost pixels per DLTile (overlap among tiles)
-
-        :return: A DLTile GeoJSON Feature
-        :rtype: DotDict
+        This method has been removed and is no longer included in the documentation.
         """
         raise NotImplementedError(
             "dltile_from_latlon is removed, use descarteslabs.scenes.DLTile.from_latlon instead"
@@ -819,19 +797,7 @@ class Raster(Service):
 
     def dltiles_from_shape(self, resolution, tilesize, pad, shape):
         """
-        Iterates over all DLTiles as GeoJSONs features that intersect the
-        GeoJSON geometry ``shape`` or Shapely shape object.
-        Deprecated for descarteslabs.scenes.DLTile.from_shape.
-
-        :param float resolution: Resolution of DLTile
-        :param int tilesize: Number of valid pixels per DLTile
-        :param int pad: Number of ghost pixels per DLTile (overlap among tiles)
-        :param str shape: A GeoJSON geometry or Shapely shape specifying a shape over
-            which to intersect DLTiles.
-
-        :return: An iterator over GeoJSON features representing
-            intersecting DLTiles
-        :rtype: generator(DotDict)
+        This method has been removed and is no longer included in the documentation.
         """
         raise NotImplementedError(
             "dltiles_from_shape is removed, use descarteslabs.scenes.DLTile.from_shape instead"
@@ -839,22 +805,7 @@ class Raster(Service):
 
     def iter_tiles_from_shape(self, resolution, tilesize, pad, shape):
         """
-        Return a feature collection of DLTile GeoJSONs that intersect
-        a GeoJSON geometry ``shape`` or a Shapely shape object.
-        Deprecated for descarteslabs.scenes.DLTile.iter_from_shape.
-
-        :param float resolution: Resolution of DLTile
-        :param int tilesize: Number of valid pixels per DLTile
-        :param int pad: Number of ghost pixels per DLTile (overlap among tiles)
-        :param str shape: A GeoJSON geometry specifying a shape over
-            which to intersect DLTiles
-
-        :return: GeoJSON FeatureCollection of intersecting DLTile geometries.
-        :rtype: DotDict
-
-        :raises descarteslabs.client.exceptions.BadRequestError: if the given
-            parameters would generate too many tiles - use
-            :meth:`Raster.iter_dltiles_from_shape` to iterate over more
+        This method has been removed and is no longer included in the documentation.
         """
         raise NotImplementedError(
             "iter_tiles_from_shape is removed, use descarteslabs.scenes.DLTile.iter_from_shape instead"
