@@ -1,4 +1,4 @@
-# Copyright 2018-2019 Descartes Labs.
+# Copyright 2018-2020 Descartes Labs.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,6 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .service import Service, JsonApiService, ThirdPartyService, NotFoundError
+from .service import (
+    Service,
+    Session,
+    JsonApiService,
+    JsonApiSession,
+    ThirdPartyService,
+    NotFoundError,
+)
 
-__all__ = ["Service", "JsonApiService", "ThirdPartyService", "NotFoundError"]
+__all__ = [
+    "Service",
+    "Session",
+    "JsonApiService",
+    "JsonApiSession",
+    "ThirdPartyService",
+    "NotFoundError",
+]
