@@ -1,4 +1,4 @@
-# Copyright 2018-2019 Descartes Labs.
+# Copyright 2018-2020 Descartes Labs.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -60,7 +60,7 @@ class Collection(object):
             if isinstance(idx, slice):
                 idx = list(range(*idx.indices(len(self))))
             if isinstance(item, six.string_types) or not isinstance(
-                item, collections.Sequence
+                item, collections.abc.Sequence
             ):  # if scalar
                 item = [item] * len(idx)
             item = list(item)
