@@ -1,4 +1,4 @@
-# Copyright 2018-2019 Descartes Labs.
+# Copyright 2018-2020 Descartes Labs.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -43,15 +43,18 @@ import shapely.geometry
 import six
 import threading
 import warnings
+import math
 
 from six.moves import reprlib
 
 from descarteslabs.client.addons import mercantile
-from descarteslabs.client.services.raster import Raster
 from descarteslabs.common import shapely_support
+from descarteslabs.common.dltile import Tile, Grid
 
 from . import _helpers
 
+EARTH_CIRCUMFERENCE_WGS84 = 2 * math.pi * 6378137
+
 
 class GeoContext(object):
     """
@@ -175,7 +178,8 @@ class AOI(GeoContext):
         geometry: GeoJSON-like dict, object with ``__geo_interface__``; optional
             Clip scenes to this geometry.
             Coordinates must be WGS84 (lat-lon).
-            If :const:`None`, scenes will just be clipped to `bounds`.
+            If :const:`None`, scenes will just be clipped to
+            :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`.
         resolution: float, optional
             Distance, in units of the CRS, that the edge of each pixel
             represents on the ground.
@@ -201,11 +205,15 @@ class AOI(GeoContext):
             native resolution and projection.
         bounds: 4-tuple, optional
             Clip scenes to these ``(min_x, min_y, max_x, max_y)`` bounds,
-            expressed in `bounds_crs` (which defaults to WGS84 lat-lon).
-            `bounds` are automatically computed from `geometry` if not specified.
-            Otherwise, `bounds` are required.
+            expressed in :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds_crs`
+            (which defaults to WGS84 lat-lon).
+            :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`
+            are automatically computed from `geometry` if not specified.
+            Otherwise,
+            :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds` are required.
         bounds_crs: str, optional, default "EPSG:4326"
-            The Coordinate Reference System of the `bounds`,
+            The Coordinate Reference System of the
+            :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`,
             given as an EPSG code (like :const:`EPSG:4326`), a PROJ.4 definition,
             or an OGC CRS Well-Known Text string.
         shape: 2-tuple, optional
@@ -228,7 +236,8 @@ class AOI(GeoContext):
         """
         shapely geometry: Clip scenes to this geometry
         Coordinates must be WGS84 (lat-lon).
-        If :const:`None`, scenes will just be clipped to `bounds`.
+        If :const:`None`, scenes will just be clipped to
+        :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`.
         """
 
         return self._geometry
@@ -278,7 +287,8 @@ class AOI(GeoContext):
     def bounds(self):
         """
         tuple: Clip scenes to these ``(min_x, min_y, max_x, max_y)`` bounds,
-        expressed in the coordinate reference system in `bounds_crs`.
+        expressed in the coordinate reference system in
+        :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds_crs`.
         """
 
         return self._bounds
@@ -286,7 +296,8 @@ class AOI(GeoContext):
     @property
     def bounds_crs(self):
         """
-        str: The coordinate reference system of the `bounds`,
+        str: The coordinate reference system of the
+        :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`,
         given as an EPSG code (like :const:`EPSG:4326`), a PROJ.4 definition,
         or an OGC CRS Well-Known Text string.
         """
@@ -310,7 +321,9 @@ class AOI(GeoContext):
         :class:`~descarteslabs.client.services.raster.raster.Raster.ndarray` or
         :class:`~descarteslabs.client.services.raster.raster.Raster.raster`.
 
-        Raises ValueError if `bounds`, `crs`, `bounds_crs`,
+        Raises ValueError if
+        :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`, `crs`,
+        :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds_crs`,
         `resolution`, or `align_pixels` is :const:`None`.
         """
 
@@ -351,9 +364,14 @@ class AOI(GeoContext):
     @property
     def __geo_interface__(self):
         """
-        dict: `self.geometry` as a GeoJSON Geometry dict,
-        otherwise `bounds` as a GeoJSON Polygon dict if `geometry` is
-        :const:`None` and `bounds_crs` is :const:`EPSG:4326`, otherwise
+        dict: :py:attr:`~descarteslabs.scenes.geocontext.AOI.geometry` as a GeoJSON Geometry dict,
+        otherwise
+        :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`
+        as a GeoJSON Polygon dict if
+        :py:attr:`~descarteslabs.scenes.geocontext.AOI.geometry` is
+        :const:`None` and
+        :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds_crs`
+        is :const:`EPSG:4326`, otherwise
         raises :exc:`RuntimeError`.
         """
 
@@ -386,12 +404,16 @@ class AOI(GeoContext):
         Note
         ----
             If you are assigning a new geometry and want bounds to updated as
-            well, use ``bounds="update"``. This will also change `bounds_crs`
+            well, use ``bounds="update"``. This will also change
+            :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds_crs`
             to :const:`EPSG:4326`, since the geometry's coordinates are in WGS84
             decimal degrees, so the new bounds determined from those coordinates
             must be in that CRS as well.
 
-            If you assign `geometry` without changing `bounds`,
+            If you assign
+            :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`
+            without changing
+            :py:attr:`~descarteslabs.scenes.geocontext.AOI.bounds`,
             the new AOI GeoContext will produce rasters with the same
             shape and covering the same spatial area as the old one, just with
             pixels masked out that fall outside your new geometry.
@@ -566,6 +588,39 @@ class DLTile(GeoContext):
     DLTiles allow you to define a grid of arbitrary spacing, resolution,
     and overlap that can cover the globe.
     DLTiles are always in a UTM projection.
+
+    Example
+    -------
+    >>> import descarteslabs as dl
+    >>> from descarteslabs.scenes import DLTile
+    >>> tile = DLTile.from_latlon(
+    ...    lat=35.691,
+    ...    lon=-105.944,
+    ...    tilesize=512,
+    ...    resolution=10,
+    ...    pad=0
+    ... )
+    >>> scenes, ctx = dl.scenes.search(tile, "landsat:LC08:PRE:TOAR")  # doctest: +SKIP
+    >>> Scenes  # doctest: +SKIP
+        SceneCollection of 93 scenes
+          * Dates: Apr 19, 2013 to Apr 14, 2017
+          * Products: landsat:LC08:PRE:TOAR: 93
+    >>> ctx     # doctest: +SKIP
+    DLTile(key='512:0:10.0:13:-17:771',
+       resolution=10.0,
+       tilesize=512,
+       pad=0,
+       crs='EPSG:32613',
+       bounds=(412960.0, 3947520.0, 418080.0, 3952640.0),
+       bounds_crs='EPSG:32613',
+       geometry=<shapely.geom...x7f121488c890>,
+       zone=13,
+       ti=-17,
+       tj=771,
+       geotrans=[
+      412960.0,...  0,
+      -10.0
+    ], ...
     """
 
     __slots__ = (
@@ -587,14 +642,18 @@ class DLTile(GeoContext):
 
     def __init__(self, dltile_dict):
         """
-        ``__init__`` instantiates a DLTile from a dict returned by `Raster.dltile`.
-
-        It's preferred to use the `DLTile.from_latlon`, `DLTile.from_shape`,
+        Constructs a DLTile from a parameter dictionary.
+        It is preferred to use the `DLTile.from_latlon`, `DLTile.from_shape`,
         or `DLTile.from_key` class methods to construct a DLTile GeoContext.
         """
 
         super(DLTile, self).__init__()
-        self._geometry = shapely.geometry.shape(dltile_dict["geometry"])
+
+        if isinstance(dltile_dict["geometry"], shapely.geometry.polygon.Polygon):
+            self._geometry = dltile_dict["geometry"]
+        else:
+            self._geometry = shapely.geometry.shape(dltile_dict["geometry"])
+
         properties = dltile_dict["properties"]
         self._key = properties["key"]
         self._resolution = properties["resolution"]
@@ -613,7 +672,7 @@ class DLTile(GeoContext):
         self._wkt = properties.get("wkt", None)
 
     @classmethod
-    def from_latlon(cls, lat, lon, resolution, tilesize, pad, raster_client=None):
+    def from_latlon(cls, lat, lon, resolution, tilesize, pad):
         """
         Return a DLTile GeoContext that covers a latitude/longitude.
 
@@ -633,23 +692,40 @@ class DLTile(GeoContext):
         pad : int
             Number of extra pixels by which each side of the tile is buffered.
             This determines the number of pixels by which two tiles overlap.
-        raster_client : descarteslabs.client.services.Raster, optional, default None
-            Unneeded in general use; lets you use a specific client instance
-            with non-default auth and parameters.
 
         Returns
         -------
         tile : DLTile
+
+        Example
+        -------
+        >>> from descarteslabs.scenes import DLTile
+        >>> # make a tile with total size 100, centered on lat, lon
+        >>> # total tilesize == tilesize + 2 * pad
+        >>> params = {
+        ...    "lat": 30.0131,
+        ...    "lon": 31.2089,
+        ...    "resolution": 10,
+        ...    "tilesize": 2,
+        ...    "pad": 49,
+        ... }
+        >>> tile = DLTile.from_latlon(**params)
+        >>> tile.key
+        '2:49:10.0:36:-8637:166079'
+        >>> tile.geometry.centroid.xy  # doctest: +SKIP
+        (array('d', [31.20899205942612]), array('d', [30.013121672688087]))
         """
 
-        if raster_client is None:
-            raster_client = Raster()
-        tile = raster_client.dltile_from_latlon(lat, lon, resolution, tilesize, pad)
-        return cls(tile)
+        grid = Grid(
+            resolution=resolution,
+            tilesize=tilesize,
+            pad=pad
+        )
+        tile = grid.tile_from_lonlat(lat=lat, lon=lon)
+        return cls(tile.geocontext)
 
     @classmethod
-    def from_shape(cls, shape, resolution, tilesize, pad, raster_client=None):
-        # TODO : non-overlapping tiles across UTM zones
+    def from_shape(cls, shape, resolution, tilesize, pad, keys_only=False):
         """
         Return a list of DLTiles that intersect the given geometry.
 
@@ -665,28 +741,124 @@ class DLTile(GeoContext):
         pad : int
             Number of extra pixels by which each side of the tile is buffered.
             This determines the number of pixels by which two tiles overlap.
-        raster_client : `Raster`, optional, default :const:`None`.
-            Unneeded in general use; lets you use a specific client instance
-            with non-default auth and parameters.
+        keys_only : bool, default False
+            Whether to return DLTile objects or only DLTile keys. Set to True when
+            returning a large number of tiles and you do not need the full objects.
 
         Returns
         -------
-        tiles : List[DLTile]
+        tiles : List[DLTile] or List[Str]
+
+        Example
+        -------
+        >>> from descarteslabs.scenes import DLTile
+        >>> shape = {
+        ... "type":"Feature",
+        ... "geometry":{
+        ...     "type":"Polygon",
+        ...     "coordinates":[[
+        ...            [-122.51140471760839,37.77130087547876],
+        ...            [-122.45475646845254,37.77475476721895],
+        ...            [-122.45303985468301,37.76657207194229],
+        ...            [-122.51057242081689,37.763446782666094],
+        ...            [-122.51140471760839,37.77130087547876]]]
+        ...    },"properties": None
+        ... }
+        >>> tiles = DLTile.from_shape(
+        ...    shape=shape,
+        ...    resolution=1,
+        ...    tilesize=500,
+        ...    pad=0,
+        ... )
+        >>> len(tiles)
+        31
         """
 
-        if raster_client is None:
-            raster_client = Raster()
-
-        if hasattr(shape, "__geo_interface__"):
-            shape = shape.__geo_interface__
-
-        tiles_fc = raster_client.dltiles_from_shape(
-            resolution=resolution, tilesize=tilesize, pad=pad, shape=shape
+        grid = Grid(
+            resolution=resolution,
+            tilesize=tilesize,
+            pad=pad
         )
-        return [cls(tile) for tile in tiles_fc["features"]]
+
+        if grid._estimate_ntiles_from_shape(shape) > 50000:
+            warnings.warn(
+                "DLTile.from_shape will return a large number of tiles. "
+                "Consider using DLTile.iter_from_shape instead."
+            )
+
+        tiles = grid.tiles_from_shape(shape=shape, keys_only=keys_only)
+        if keys_only:
+            result = [tile for tile in tiles]
+        else:
+            result = [cls(tile.geocontext) for tile in tiles]
+        return result
 
     @classmethod
-    def from_key(cls, dltile_key, raster_client=None):
+    def iter_from_shape(cls, shape, resolution, tilesize, pad, keys_only=False):
+        """
+        Return a iterator for DLTiles that intersect the given geometry.
+
+        Parameters
+        ----------
+        shape : GeoJSON-like
+            A GeoJSON dict, or object with a ``__geo_interface__``. Must be in
+            :const:`EPSG:4326` (WGS84 lat-lon) projection.
+        resolution : float
+            Distance, in meters, that the edge of each pixel represents on the ground.
+        tilesize : int
+            Length of each side of the tile, in pixels.
+        pad : int
+            Number of extra pixels by which each side of the tile is buffered.
+            This determines the number of pixels by which two tiles overlap.
+        keys_only : bool, default False
+            Whether to return DLTile objects or only DLTile keys. Set to True when
+            returning a large number of tiles and you do not need the full objects.
+
+        Returns
+        -------
+        Iterator of DLTiles or str
+
+        Example
+        -------
+        >>> from descarteslabs.scenes import DLTile
+        >>> shape = {
+        ... "type":"Feature",
+        ... "geometry":{
+        ...     "type":"Polygon",
+        ...     "coordinates":[[
+        ...            [-122.51140471760839,37.77130087547876],
+        ...            [-122.45475646845254,37.77475476721895],
+        ...            [-122.45303985468301,37.76657207194229],
+        ...            [-122.51057242081689,37.763446782666094],
+        ...            [-122.51140471760839,37.77130087547876]]]
+        ...    },"properties": None
+        ... }
+        >>> gen = DLTile.from_shape(
+        ...    shape=shape,
+        ...    resolution=1,
+        ...    tilesize=500,
+        ...    pad=0,
+        ...    keys_only=True
+        ... )
+        >>> tiles = [tile for tile in gen]  # doctest: +SKIP
+        >>> tiles[0]                        # doctest: +SKIP
+        '500:0:1.0:10:94:8359'
+        """
+
+        grid = Grid(
+            resolution=resolution,
+            tilesize=tilesize,
+            pad=pad
+        )
+        tiles = grid.tiles_from_shape(shape=shape, keys_only=keys_only)
+        for tile in tiles:
+            if keys_only:
+                yield tile
+            else:
+                yield cls(tile.geocontext)
+
+    @classmethod
+    def from_key(cls, dltile_key):
         """
         Return a DLTile GeoContext from a DLTile key.
 
@@ -694,32 +866,175 @@ class DLTile(GeoContext):
         ----------
         dltile_key : str
             DLTile key, e.g. '128:16:960.0:15:-1:37'
-        raster_client : `Raster`, optional, default :const:`None`.
-            Unneeded in general use; lets you use a specific client instance
-            with non-default auth and parameters.
 
         Returns
         -------
         tile: DLTile
+
+        Example
+        -------
+        >>> from descarteslabs.scenes import DLTile
+        >>> tile = DLTile.from_key("2048:16:30.0:15:3:80")
+        >>> tile            # doctest: +SKIP
+        DLTile(key='2048:16:30.0:15:3:80',
+               resolution=30.0,
+               tilesize=2048,
+               pad=16,
+               crs='EPSG:32615',
+               bounds=(683840.0, 4914720.0, 746240.0, 4977120.0),
+               bounds_crs='EPSG:32615',
+               geometry=<shapely.geom...>,
+               zone=15,
+               ti=3,
+               tj=80,
+               geotrans=[
+        ...
         """
 
-        if raster_client is None:
-            raster_client = Raster()
-        tile = raster_client.dltile(dltile_key)
-        return cls(tile)
+        tile = Tile.from_key(dltile_key)
+        return cls(tile.geocontext)
 
-    def assign(self, pad, raster_client=None):
+    def subtile(self, subdivide, resolution=None, pad=None, keys_only=False):
         """
-        Return a copy of the DLTile with the pad value modified.
+        Return an iterator for new DLTiles that subdivide this tile.
+
+        The DLtile will be sub-divided into subdivide^2 total sub-tiles each with a side length
+        of tile_size / subdivide. The resulting sub-tile size must be an integer.
+        Each sub-tile will by default inherit the same resolution and pad as the orginal tile.
+
+        Parameters
+        ----------
+        subdivide : int
+            The value to subdivide the tile. The total number of sub-tiles will be the
+            square of this value. This value must evenly divide the original tilesize.
+        resolution : None, float
+            A new resolution for the sub-tiles. None defaults to the original DLTile resolution.
+            The new resolution must evenly divide the the original tilesize divided by
+            the subdivide ratio.
+        pad : None, int
+            A new pad value for the sub-tiles. None defaults to the original DLTile pad value.
+        keys_only : bool, default False
+            Whether to return DLTile objects or only DLTile keys. Set to True when returning a large number of tiles
+            and you do not need the full objects.
 
         Returns
         -------
-        new : `DLTile`
+        Iterator over DLTiles or str
+
+        Example:
+        -------
+        >>> from descarteslabs.scenes import DLTile
+        >>> tile = DLTile.from_key("2048:0:30.0:15:3:80")
+        >>> tiles = [tile for tile in tile.subtile(8)]
+        >>> len(tiles)
+        64
+        >>> tiles[0].tilesize
+        256
         """
 
-        key = self._key.split(":")
-        key[1] = str(int(pad))
-        return self.from_key(":".join(key), raster_client=raster_client)
+        subtiles = Tile.from_key(self.key).subtile(
+            subdivide=subdivide,
+            new_resolution=resolution,
+            new_pad=pad,
+        )
+
+        for tile in subtiles:
+            if keys_only:
+                yield tile.key
+            else:
+                yield DLTile(tile.geocontext)
+
+    def rowcol_to_latlon(self, row, col):
+        """
+        Convert pixel coordinates to lat, lon coordinates
+
+        Parameters
+        ----------
+        row : int or List[int]
+            Pixel row coordinate or coordinates
+        col : int or List[int]
+            Pixel column coordinate or coordinates
+
+        Returns
+        -------
+        coords : List[Tuple[float], Tuple[float]]
+            List with the first element the latitude values and the second element longitude values
+
+        Example
+        -------
+        >>> from descarteslabs.scenes import DLTile
+        >>> tile = DLTile.from_key("2048:0:30.0:15:3:80")
+        >>> tile.rowcol_to_latlon(row=56, col=1111)
+        [(44.89479253978484,), (-90.24352536949974,)]
+        """
+
+        lonlat = Tile.from_key(self.key).rowcol_to_lonlat(row=row, col=col)
+        lonlat = lonlat.tolist()
+        if isinstance(lonlat[0], (int, float)):
+            result = [(lonlat[1],), (lonlat[0],)]
+        else:
+            result = list(zip(*lonlat))
+            result[0], result[1] = result[1], result[0]
+        return result
+
+    def latlon_to_rowcol(self, lat, lon):
+        """
+        Convert lat, lon coordinates to pixel coordinates
+
+        Parameters
+        ----------
+        lat: float or List[float]
+            Latitude coordinate or coordinates
+        lon: float or List[float]
+            Longitude coordinate or coordinates
+
+        Returns
+        -------
+        coords: List[Tuple[int] Tuple[int]]
+            Tuple with the first element the row values and the second element column values
+
+        Example
+        -------
+        >>> from descarteslabs.scenes import DLTile
+        >>> tile = DLTile.from_key("2048:0:30.0:15:3:80")
+        >>> tile.latlon_to_rowcol(lat=44.8, lon=-90.2)
+        [(403,), (1238,)]
+        """
+
+        rowcol = Tile.from_key(self.key).lonlat_to_rowcol(lat=lat, lon=lon)
+        rowcol = rowcol.tolist()
+        if isinstance(rowcol[0], (int, float)):
+            result = [(rowcol[0],), (rowcol[1],)]
+        else:
+            result = list(zip(*rowcol))
+        return result
+
+    def assign(self, pad):
+        """
+        Return a copy of the DLTile with the pad value modified.
+
+        Parameters
+        ----------
+        pad : int
+            New pad value
+
+        Returns
+        -------
+        tile : DLTile
+
+        Example:
+        --------
+        >>> from descarteslabs.scenes import DLTile
+        >>> tile = DLTile.from_key("2048:16:30.0:15:3:80")
+        >>> tile.pad
+        16
+        >>> tile = tile.assign(123)
+        >>> tile.pad
+        123
+        """
+
+        tile = Tile.from_key(self.key).assign(pad=pad)
+        return DLTile(tile.geocontext)
 
     @property
     def key(self):
@@ -732,7 +1047,7 @@ class DLTile(GeoContext):
 
     @property
     def resolution(self):
-        "float: Distance, in meters, that the edge of each pixel represents on the ground"
+        """float: Distance, in meters, that the edge of each pixel represents on the ground"""
 
         return self._resolution
 
@@ -741,11 +1056,20 @@ class DLTile(GeoContext):
         """
         int: Length of each side of the tile, in pixels.
         Note that the total number of pixels along each side of an image is
-        ``tile_size + 2*padding``
+        ``tile_size + 2 * padding``
         """
 
         return self._tilesize
 
+    @property
+    def tile_extent(self):
+        """
+        int: total extent of geocontext length in pixels, including pad.
+        Size is ``tile_size + 2 * pad``.
+        """
+
+        return self._tilesize + 2 * self._pad
+
     @property
     def pad(self):
         """
@@ -768,7 +1092,8 @@ class DLTile(GeoContext):
     def bounds(self):
         """
         tuple: The ``(min_x, min_y, max_x, max_y)`` of the area covered by
-        this DLTile, in the UTM coordinate reference system given in `bounds_crs`.
+        this DLTile, in the UTM coordinate reference system given in
+        :py:attr:`~descarteslabs.scenes.geocontext.DLTile.bounds_crs`.
         """
 
         return self._bounds
@@ -776,7 +1101,8 @@ class DLTile(GeoContext):
     @property
     def bounds_crs(self):
         """
-        str: The coordinate reference system of the `bounds`,
+        str: The coordinate reference system of the
+        :py:attr:`~descarteslabs.scenes.geocontext.DLTile.bounds`,
         given as an EPSG code (like :const:`EPSG:32615`).
         A DLTile's CRS is always UTM.
         """
@@ -794,19 +1120,19 @@ class DLTile(GeoContext):
 
     @property
     def zone(self):
-        "int: The UTM zone of this tile"
+        """int: The UTM zone of this tile"""
 
         return self._zone
 
     @property
     def ti(self):
-        "int: The y-index of this tile in its grid"
+        """int: The y-index of this tile in its grid"""
 
         return self._ti
 
     @property
     def tj(self):
-        "int: The x-index of this tile in its grid"
+        """int: The x-index of this tile in its grid"""
 
         return self._tj
 
@@ -848,19 +1174,19 @@ class DLTile(GeoContext):
 
     @property
     def proj4(self):
-        "str: PROJ.4 definition for this DLTile's coordinate reference system"
+        """str: PROJ.4 definition for this DLTile's coordinate reference system"""
 
         return self._proj4
 
     @property
     def wkt(self):
-        "str: OGC Well-Known Text definition for this DLTile's coordinate reference system"
+        """str: OGC Well-Known Text definition for this DLTile's coordinate reference system"""
 
         return self._wkt
 
     @property
     def __geo_interface__(self):
-        "dict: `geometry` as a GeoJSON Polygon"
+        """dict: :py:attr:`~descarteslabs.scenes.geocontext.DLTile.geometry` as a GeoJSON Polygon"""
 
         with self._geometry_lock_:
             # see comment in `GeoContext.__init__` for why we need to prevent
@@ -957,7 +1283,8 @@ class XYZTile(GeoContext):
     @property
     def bounds_crs(self):
         """
-        str: The coordinate reference system of the `bounds`.
+        str: The coordinate reference system of the
+        :py:attr:`~descarteslabs.scenes.geocontext.XYZTile.bounds`.
         Always :const:`EPSG:3857` (spherical Mercator, aka "Web Mercator")
         """
 
@@ -971,9 +1298,18 @@ class XYZTile(GeoContext):
 
         return 256
 
+    @property
+    def resolution(self):
+        """
+        float: Distance, in meters, that the edge of each pixel represents in the
+        spherical Mercator ("Web Mercator", EPSG:3857) projection.
+        """
+        num_tiles = 1 << self.z
+        return EARTH_CIRCUMFERENCE_WGS84 / num_tiles / self.tilesize
+
     @property
     def __geo_interface__(self):
-        "dict: `geometry` as a GeoJSON Polygon"
+        "dict: :py:attr:`~descarteslabs.scenes.geocontext.XYZTile.geometry` as a GeoJSON Polygon"
 
         return self.geometry.__geo_interface__
 
@@ -989,5 +1325,5 @@ class XYZTile(GeoContext):
             "srs": self.crs,
             "bounds_srs": self.bounds_crs,
             "align_pixels": False,
-            "dimensions": (self.tilesize, self.tilesize),
+            "resolution": self.resolution,
         }
