@@ -210,7 +210,7 @@ class SceneCollection(Collection):
         NotFoundError
             If a Scene's ID cannot be found in the Descartes Labs catalog
         BadRequestError
-            If the Descartes Labs platform is given unrecognized parameters
+            If the Descartes Labs Platform is given unrecognized parameters
         """
         if len(self) == 0:
             raise ValueError("This SceneCollection is empty")
@@ -419,7 +419,7 @@ class SceneCollection(Collection):
         NotFoundError
             If a Scene's ID cannot be found in the Descartes Labs catalog
         BadRequestError
-            If the Descartes Labs platform is given unrecognized parameters
+            If the Descartes Labs Platform is given unrecognized parameters
         """
         if len(self) == 0:
             raise ValueError("This SceneCollection is empty")
@@ -435,6 +435,7 @@ class SceneCollection(Collection):
         (bands, scaling, mask_alpha, drop_alpha) = self._mask_alpha_if_applicable(
             bands, mask_alpha=mask_alpha, scaling=scaling
         )
+        mask_nodata = bool(mask_nodata)
 
         scales, data_type = _scaling.multiproduct_scaling_parameters(
             self._product_band_properties(), bands, scaling, data_type
@@ -448,6 +449,10 @@ class SceneCollection(Collection):
             data_type=data_type,
             resampler=resampler,
             processing_level=processing_level,
+            mask_nodata=mask_nodata,
+            mask_alpha=mask_alpha,
+            drop_alpha=drop_alpha,
+            masked=mask_nodata or mask_alpha,
             **raster_params
         )
         try:
@@ -472,16 +477,6 @@ class SceneCollection(Collection):
             # if only 1 band requested, still return a 3d array
             arr = arr[np.newaxis]
 
-        if mask_nodata or mask_alpha:
-            if mask_alpha:
-                alpha = arr[-1]
-                if drop_alpha:
-                    arr = arr[:-1]
-                    bands.pop(-1)
-                arr.mask = ~alpha.astype(bool)
-        else:
-            arr = arr.data
-
         if bands_axis != 0:
             arr = np.moveaxis(arr, 0, bands_axis)
         if raster_info:
@@ -600,7 +595,7 @@ class SceneCollection(Collection):
         NotFoundError
             If a Scene's ID cannot be found in the Descartes Labs catalog
         BadRequestError
-            If the Descartes Labs platform is given unrecognized parameters
+            If the Descartes Labs Platform is given unrecognized parameters
         """
         if len(self) == 0:
             raise ValueError("This SceneCollection is empty")
@@ -706,6 +701,7 @@ class SceneCollection(Collection):
         scaling=None,
         data_type=None,
         mask_alpha=None,
+        nodata=None,
     ):
         """
         Download all scenes as a single image file.
@@ -720,7 +716,7 @@ class SceneCollection(Collection):
             or a sequence of band names (``["red", "green", "blue"]``).
         ctx : :class:`~descarteslabs.scenes.geocontext.GeoContext`
             A :class:`~descarteslabs.scenes.geocontext.GeoContext` to use when loading the Scenes
-        dest : str, path-like object, or file-like object, default None
+        dest : str or path-like object, default None
             Where to write the image file.
 
             * If None (default), it's written to an image file of the given ``format``
@@ -734,9 +730,8 @@ class SceneCollection(Collection):
 
               Note that path-like objects (such as pathlib.Path) are only supported
               in Python 3.6 or later.
-            * If a file-like object, it's written into that file.
         format : str, default "tif"
-            If a file-like object or None is given as ``dest``: one of "tif", "png", or "jpg".
+            If None is given as ``dest``: one of "tif", "png", or "jpg".
 
             If a str or path-like object is given as ``dest``, ``format`` is ignored
             and determined from the extension on the path (one of ".tif", ".png", or ".jpg").
@@ -762,6 +757,8 @@ class SceneCollection(Collection):
             If the alpha band is available for all scenes in the collection and
             ``mask_alpha`` is None, ``mask_alpha`` is set to True. If not,
             mask_alpha is set to False.
+        nodata : None, number
+            NODATA value for a geotiff file. Will be assigned to any masked pixels.
 
         Returns
         -------
@@ -778,8 +775,6 @@ class SceneCollection(Collection):
         'mosaic-nir-red.jpg'
         >>> scenes.download_mosaic("nir red", tile, dest="mosaics/{}.png".format(tile.key))  # doctest: +SKIP
         'mosaics/256:0:75.0:15:-5:230.png'
-        >>> with open("another_mosaic.jpg", "wb") as f:
-        ...     scenes.download_mosaic("swir2", tile, dest=f, format="jpg")  # doctest: +SKIP
 
 
         Raises
@@ -793,7 +788,7 @@ class SceneCollection(Collection):
         NotFoundError
             If a Scene's ID cannot be found in the Descartes Labs catalog
         BadRequestError
-            If the Descartes Labs platform is given unrecognized parameters
+            If the Descartes Labs Platform is given unrecognized parameters
         """
         if len(self) == 0:
             raise ValueError("This SceneCollection is empty")
@@ -806,8 +801,6 @@ class SceneCollection(Collection):
             self._product_band_properties(), bands, scaling, data_type
         )
 
-        print(self.each.properties["id"].combine())
-
         return _download._download(
             inputs=self.each.properties["id"].combine(),
             bands_list=bands,
@@ -818,6 +811,7 @@ class SceneCollection(Collection):
             format=format,
             resampler=resampler,
             processing_level=processing_level,
+            nodata=nodata,
             raster_client=self._raster_client,
         )
 
@@ -915,8 +909,11 @@ class SceneCollection(Collection):
         alpha_band_name = "alpha"
         if isinstance(mask_alpha, str):
             alpha_band_name = mask_alpha
+            mask_alpha = True
         elif mask_alpha is None:
             mask_alpha = self._collection_has_alpha(alpha_band_name)
+        elif type(mask_alpha) is not bool:
+            raise ValueError("'mask_alpha' must be None, a band name, or a bool.")
 
         drop_alpha = False
         if mask_alpha:
