@@ -1,9 +1,11 @@
 import six
 
+from . import widgets
 
 try:
     from .layer import WorkflowsLayer
     from .layer_controller import LayerController, LayerControllerList
+    from .layer_picker import ImagePickerWidget, ImageCollectionPickerWidget
     from .map_ import MapApp, Map
     from .parameters import ParameterSet, ProxytypeInstance
     from .inspector import PixelInspector
@@ -29,14 +31,17 @@ try:
     Browse shared `Workflow` objects (and add them to the map) with a predefined `WorkflowsBrowser` instance.
     """
 
+    _successful_import = True
 except Exception as e:
+    _successful_import = False
     error = e
 
     class MissingImports(object):
         def _error(self, *args, **kwargs):
             err = ImportError(
-                "Optional dependencies needed for map visualization and Workflow browsing are missing.\n"
-                "Please install `ipyleaflet`, or better:\n"
+                "Optional dependencies needed for map visualization and Workflow browsing "
+                "are missing or outdated.\n"
+                "Please install `ipyleaflet` and `traitlets`, or better:\n"
                 "$ pip install --upgrade 'descarteslabs[complete]'\n"
                 "Then, install the Jupyter extensions:\n"
                 "$ jupyter labextension install jupyter-leaflet @jupyter-widgets/jupyterlab-manager\n"
@@ -52,6 +57,8 @@ except Exception as e:
     WorkflowsLayer = map
     LayerController = map
     LayerControllerList = map
+    ImagePickerWidget = map
+    ImageCollectionPickerWidget = map
     MapApp = map
     Map = map
     ParameterSet = map
@@ -64,6 +71,8 @@ __all__ = [
     "WorkflowsLayer",
     "LayerController",
     "LayerControllerList",
+    "ImagePickerWidget",
+    "ImageCollectionPickerWidget",
     "Map",
     "MapApp",
     "map",
@@ -72,4 +81,5 @@ __all__ = [
     "PixelInspector",
     "WorkflowsBrowser",
     "flows",
+    "widgets",
 ]
