@@ -91,6 +91,7 @@ class Job:
         _ruster=None,
         _trace=False,
         client=None,
+        num_retries=None,
         **arguments,
     ):
         """
@@ -117,6 +118,12 @@ class Job:
         client: `.workflows.client.Client`, optional
             Allows you to use a specific client instance with non-default
             auth and parameters
+        num_retries: Int, optional
+            The number of retries to make in the event of a request failure. If you are making numerous long-running
+            asynchronous requests, you can use this parameter as a way to indicate that you are comfortable waiting
+            and retrying in response to RESOURCE EXHAUSTED errors. By default, most failures will trigger a small number
+            of retries, but if you have reached your outstanding job limit, by default, the client will not retry.
+            See the `compute section of the Workflows Guide </guides/workflows/compute.html>` for more information.
         **arguments: Any
             Values for all parameters that ``obj`` depends on
             (or arguments that ``obj`` takes, if it's a `.Function`).
@@ -164,6 +171,7 @@ class Job:
                 destination=destination_proto,
             ),
             timeout=client.DEFAULT_TIMEOUT,
+            num_retries=num_retries,
         )
 
         self._message = message
