@@ -5,12 +5,17 @@ from .exceptions import ProxyTypeError
 
 
 class Castable(object):
-    "Mixin to support casting without explicitly allowing it in ``__init__``"
+    """
+    Mixin to support casting and instantiation from grafts as separate constructors from ``__init__``
+
+    Provides methods to construct delayed objects from grafts, and to propagate along
+    any graft parameters they depend on.
+    """
 
     @classmethod
-    def _from_graft(cls, graft):
+    def _from_graft(cls, graft, params=()):
         """
-        Create an instance of this class with the given graft dict, circumventing ``__init__``
+        Create an instance of this class with the given graft dict and parameters, circumventing ``__init__``
 
         To use safely, this class must function correctly if its ``__init__`` method is not called.
 
@@ -22,19 +27,58 @@ class Castable(object):
         ), "Attempted to instantiate {} from the non-graft-like object {!r}".format(
             cls.__name__, graft
         )
+        assert isinstance(params, tuple), f"params must be a tuple, not {params!r}"
+
         # create a new, empty ``cls`` object, circumventing its __init__ method.
         new = cls.__new__(cls)
         new.graft = graft
+        new.params = params
+        return new
+
+    @classmethod
+    def _as_param(cls, name: str):
+        "Construct a new instance of `cls` as a parameter referencing `name`."
+        new = cls._from_graft(client.keyref_graft(name))
+        new.params = (new,)
+        # ^ NOTE: this is all that makes something a parameter: its params list contains itself.
+        # that way, all other objects that interact with it will propagate that parameter forward.
+        new._name = name
         return new
 
     @classmethod
     def _from_apply(cls, function, *args, **kwargs):
-        "Shorthand for ``cls._from_graft(client.apply_graft(function, *args, **kwargs))``"
-        return cls._from_graft(client.apply_graft(function, *args, **kwargs))
+        """
+        Construct a new instance of `cls` from a graft function application.
+
+        Like ``cls._from_graft(client.apply_graft(function, *args, **kwargs))``,
+        with parameter merging of the inputs.
+        """
+        return cls._from_graft(
+            client.apply_graft(function, *args, **kwargs),
+            params=merge_params(function, *args, *kwargs.values()),
+        )
 
     def _cast(self, cls):
         "Copy of ``self`` as ``cls``. ``cls.__init__`` will not be called."
-        return cls._from_graft(self.graft)
+        return cls._from_graft(self.graft, self.params)
+
+
+def merge_params(*args: Castable) -> tuple:
+    "Returns merged and deduplicated parameters from the args"
+    params = {}
+    for arg in args:
+        for p in getattr(arg, "params", ()):
+            current = params.setdefault(p._name, p)
+            if current is not p:
+                raise ValueError(
+                    f"Arguments combine two different parameter objects with the same names. "
+                    f"A parameter named {p._name!r} ({p!r}) is used by the argument {arg!r}, "
+                    f"but a prior argument used a different parameter object named {p._name!r}: "
+                    f"{current!r}.\n\n"
+                    f"If these parameters {p._name!r} mean different things, then give them different names. "
+                    f"Otherwise, use the same parameter object for both."
+                )
+    return tuple(params.values())
 
 
 class Proxytype(Castable):
@@ -113,9 +157,21 @@ class GenericProxytypeMetaclass(type):
     Override ``isinstance`` and ``issubclass`` to make them covariant
     for `GenericProxytype`s and their concrete subclasses.
 
-    Formally: ``issubclass(typA, typB) is True`` iff the generic type of ``typA``
+    Types can modify this behavior by implementing the classmethod
+    ``_issubclass(cls, other: type) -> bool``.
+    This is called on the possible supertype (second argument to ``issubclass``).
+    If both types implement ``_issubclass``, only the possible supertype's is called.
+    ``_issubclass`` is called iff both types inherit from the same generic type.
+    For example, ``Foo._issubclass`` is not called in either ``issubclass(Bar[Int], Foo[Int])``
+    or ``issubclass(int, Foo[Int])``, because the other type in question isn't also a ``Foo``.
+
+    If ``_issubclass`` is not defined on the possible supertype,
+    then the default behavior applies. Formally:
+
+    ``issubclass(typA, typB) is True`` iff the generic type of ``typA``
     is a subclass of the generic type of ``typB``, and all the type parameters of ``typA``
-    are subclasses of the equivalent type parameters of ``typB``.
+    are subclasses of---or equal to, in the case of primitive values---the equivalent
+    type parameters of ``typB``
 
     This ensures that:
 
@@ -153,6 +209,9 @@ class GenericProxytypeMetaclass(type):
 
     def __subclasscheck__(mycls, othercls):
         "Covariantly check whether ``issubclass(othercls, mycls)``"
+        if not isinstance(othercls, type):
+            raise TypeError("issubclass() arg 1 must be a class")
+
         # vanilla `issubclass`, to prevent infinite recursion into this `__subclasscheck__` method
         # (except the order of the args is reversed from `issubclass`, so call it "issuperclass" instead)
         issuperclass = type.__subclasscheck__
@@ -162,10 +221,9 @@ class GenericProxytypeMetaclass(type):
         try:
             my_generic_parent_type = mycls._generictype
         except AttributeError:
-            assert issuperclass(
-                GenericProxytype, mycls
-            ), "{} is not a subclass of GenericProxytype. Do not use the GenericProxytypeMetaclass on it!".format(
-                mycls
+            assert issuperclass(GenericProxytype, mycls), (
+                f"{mycls} is not a subclass of GenericProxytype. "
+                "Do not use the GenericProxytypeMetaclass on it!"
             )
             # If `mycls` doesn't have a `_generictype` attr, it's already a generic
             my_generic_parent_type = mycls
@@ -178,16 +236,12 @@ class GenericProxytypeMetaclass(type):
             other_generic_parent_type = othercls
 
         assert my_generic_parent_type._type_params is None, (
-            "Expected {} to be generic, "
-            "but _type_params is not None: {}".format(
-                my_generic_parent_type, my_generic_parent_type._type_params
-            )
+            f"Expected {my_generic_parent_type} to be generic, "
+            f"but _type_params is not None: {my_generic_parent_type._type_params}"
         )
         assert getattr(other_generic_parent_type, "_type_params", None) is None, (
-            "Expected {} to be generic, "
-            "but _type_params is not None: {}".format(
-                other_generic_parent_type, other_generic_parent_type._type_params
-            )
+            f"Expected {other_generic_parent_type} to be generic, "
+            f"but _type_params is not None: {other_generic_parent_type._type_params}"
         )
 
         # Check that the generic type of the other is a subclass of our generic type,
@@ -195,17 +249,19 @@ class GenericProxytypeMetaclass(type):
         # or the type parameters of other are all subclasses of our equivalent type parameters
         return issuperclass(my_generic_parent_type, other_generic_parent_type) and (
             mycls._type_params is None
-            or _type_params_issubclass(othercls._type_params, mycls._type_params)
+            or mycls._issubclass(othercls)
+            # NOTE: the default `_issubclass` is defined on `GenericProxytype`,
+            # but classes can override if they wish.
         )
 
-    def __init__(cls, name, bases, dct):
+    def __init__(cls, name, bases, dct, **kwargs):
         # Ensure every subclass gets its own `_concrete_subtypes` dict,
         # which it will use to register parameterized (concrete) subtypes
         # (so calling `List[Int]` twice will return the same object both times)
 
         # QUESTION(gabe): does it matter that this happens on the concrete subtypes themselves?
         cls._concrete_subtypes = {}
-        super(GenericProxytypeMetaclass, cls).__init__(name, bases, dct)
+        super(GenericProxytypeMetaclass, cls).__init__(name, bases, dct, **kwargs)
 
     def __getitem__(cls, idx):
         "Emulate __class_getitem__ for Python < 3.7"
@@ -215,7 +271,7 @@ class GenericProxytypeMetaclass(type):
             raise TypeError("{!r} object is not subscriptable".format(cls.__name__))
 
 
-def _type_params_issubclass(type_params, super_type_params):
+def type_params_issubclass(type_params: tuple, super_type_params: tuple) -> bool:
     """
     Whether each element in `type_params` is a subclass of the equivalent element in `super_type_params`.
 
@@ -228,14 +284,26 @@ def _type_params_issubclass(type_params, super_type_params):
     if isinstance(type_params, PRIMITIVES):
         return type_params == super_type_params
     elif isinstance(type_params, tuple):
-        return len(type_params) == len(super_type_params) and all(
-            _type_params_issubclass(cls, super_cls)
-            for cls, super_cls in zip(type_params, super_type_params)
+        return (
+            isinstance(super_type_params, tuple)
+            and len(type_params) == len(super_type_params)
+            and all(
+                type_params_issubclass(cls, super_cls)
+                for cls, super_cls in zip(type_params, super_type_params)
+            )
         )
     elif isinstance(type_params, dict):
-        return six.viewkeys(type_params) == six.viewkeys(super_type_params) and all(
-            _type_params_issubclass(cls, super_type_params[key])
-            for key, cls in six.iteritems(type_params)
+        return (
+            isinstance(super_type_params, dict)
+            and len(type_params) == len(super_type_params)
+            and all(
+                # NOTE: dict order matters!
+                type_params_issubclass(k, super_k)
+                and type_params_issubclass(v, super_v)
+                for (k, v), (super_k, super_v) in zip(
+                    type_params.items(), super_type_params.items()
+                )
+            )
         )
     else:
         return issubclass(type_params, super_type_params)
@@ -271,10 +339,22 @@ class GenericProxytype(Proxytype):
           ```
           if self._type_params is None:
               raise TypeError(
-                  f"Cannot instantiate a generic {type(self).__name__}; the item type must be specified"
+                  f"Cannot instantiate a generic {type(self).__name__}; the item type must be specified "
+                  f"like {type(self).__name__}[<example of using type params>].
               )
           ```
 
+          You can also do this with:
+          ```
+          try:
+              super().__init__()
+          except TypeError:
+              raise TypeError(
+                  f"Cannot instantiate a generic {type(self).__name__}; the item type must be specified "
+                  f"like {type(self).__name__}[<example of using type params>].
+              ) from None
+          ```
+
           If your Proxytype is representing a Python type, ``__init__`` should probably resemble
           the behavior of the Python type, and/or be able to take an instance of the Python type
           and represent it as the proxy type.
@@ -326,6 +406,11 @@ class GenericProxytype(Proxytype):
     the Python ``isinstance`` and ``issubclass`` methods will also behave covariantly for your
     custom generic type: ``issubclass(MyType[MyIntSubclass], `MyType[Int])`` will be True.
 
+    To customize the behavior of ``isinstance`` and ``issubclass`` checking, override the
+    ``_issubclass`` classmethod. By default, it checks whether corresponding elements
+    in the two class's type parameters are all subclasses, or equal. See `GenericProxytypeMetaclass`
+    for more.
+
     By implementing ``_validate_params``, you can customize the valid type parameters of a class.
     ``_validate_params`` should take a tuple of type parameters and apply appropriate validation
     to each (in most cases you will at least want to validate that each type parameter is a subclass of Proxytype).
@@ -352,8 +437,7 @@ class GenericProxytype(Proxytype):
         # and can be safely considered equivalent.
         # (`cls._concrete_subtypes` was added by the GenericProxytypeMetaclass, and is an empty dict.)
         hashable_type_params = tuple(
-            frozenset(six.iteritems(x)) if isinstance(x, dict) else x
-            for x in type_params
+            tuple(x.items()) if isinstance(x, dict) else x for x in type_params
         )
         try:
             return cls._concrete_subtypes[hashable_type_params]
@@ -366,14 +450,17 @@ class GenericProxytype(Proxytype):
                 # so we do our own dict formatting using `cls.__name__` for read/copy-ability
                 "{{{}}}".format(
                     ", ".join(
-                        "{!r}: {}".format(k, param_cls.__name__)
-                        for k, param_cls in sorted(six.iteritems(type_param))
+                        "{!r}: {}".format(
+                            k,
+                            param.__name__ if isinstance(param, type) else repr(param),
+                        )
+                        for k, param in type_param.items()
                     )
                 )
                 if isinstance(type_param, dict)
                 else type_param.__name__
                 if isinstance(type_param, type)
-                else str(type_param)
+                else repr(type_param)
                 for type_param in type_params
             )
 
@@ -402,6 +489,19 @@ class GenericProxytype(Proxytype):
             cls._concrete_subtypes[hashable_type_params] = ConcreteType
             return ConcreteType
 
+    @classmethod
+    def _issubclass(cls, other: type) -> bool:
+        """
+        Whether ``other`` is a subclass of the current class.
+
+        Overriding this method allows customization of ``isinstance`` and ``issubclass`` checks.
+
+        Note that ``_issubclass`` is only called when both classes are the same
+        *generic* type; i.e. inside this method for ``List[Int]``, you can assume that
+        ``other`` is also a subclass of ``List``.
+        """
+        return type_params_issubclass(other._type_params, cls._type_params)
+
     @classmethod
     def _promote(cls, obj):
         if isinstance(obj, cls):
@@ -413,12 +513,16 @@ class GenericProxytype(Proxytype):
             return obj.cast(cls)
         except Exception:
             pass
-        return cls(obj)
+
+        try:
+            return cls(obj)
+        except Exception as e:
+            raise ProxyTypeError(e)
 
     @classmethod
-    def _from_graft(cls, graft):
+    def _from_graft(cls, graft, params=()):
         """
-        Create an instance of this class with the given graft dict, circumventing ``__init__``
+        Create an instance of this class with the given graft dict and params, circumventing ``__init__``
 
         To use safely, this class must function correctly if its ``__init__`` method is not called.
 
@@ -428,7 +532,14 @@ class GenericProxytype(Proxytype):
         assert not is_generic(cls), "Cannot instantiate a generic {}".format(
             cls.__name__
         )
-        return super(GenericProxytype, cls)._from_graft(graft)
+        return super(GenericProxytype, cls)._from_graft(graft, params=params)
+
+    def __init__(self):
+        if self._type_params is None:
+            raise TypeError(
+                "Cannot instantiate a generic {0}; the type parameters must be provided "
+                "(like `{0}[...]`)".format(type(self).__name__)
+            )
 
 
 def is_generic(type_):
